@@ -53,14 +53,14 @@ import java.util.Set;
  * @author Doug Voet (dvoet at broadinstitute dot org)
  */
 @CommandLineProgramProperties(
-        summary = CollectInsertSizeMetrics.USAGE_SUMMARY + CollectInsertSizeMetrics.USAGE_BRIEF,
-        oneLineSummary = CollectInsertSizeMetrics.USAGE_BRIEF,
+        summary = CollectInsertSizeMetrics.USAGE_SUMMARY + CollectInsertSizeMetrics.USAGE_DETAILED,
+        oneLineSummary = CollectInsertSizeMetrics.USAGE_SUMMARY,
         programGroup = DiagnosticsAndQCProgramGroup.class
 )
 @DocumentedFeature
 public class CollectInsertSizeMetrics extends SinglePassSamProgram {
-    static final String USAGE_BRIEF = "Collect metrics about the insert size distribution of a paired-end library.";
-    static final String USAGE_SUMMARY = "<p>This tool provides useful metrics for validating library construction including " +
+    static final String USAGE_SUMMARY = "Collect metrics about the insert size distribution of a paired-end library. ";
+    static final String USAGE_DETAILED = "This tool provides useful metrics for validating library construction including " +
             "the insert size distribution and read orientation of paired-end libraries.</p>" +
             "" +
             "The expected proportions of these metrics vary depending on the type of library preparation used, resulting from " +
@@ -128,11 +128,16 @@ public class CollectInsertSizeMetrics extends SinglePassSamProgram {
      */
     @Override
     protected String[] customCommandLineValidation() {
-         if (MINIMUM_PCT < 0 || MINIMUM_PCT > 0.5) {
-             return new String[]{"MINIMUM_PCT was set to " + MINIMUM_PCT + ". It must be between 0 and 0.5 so all data categories don't get discarded."};
-         }
+        final List<String> errorMsgs = new ArrayList<String>();
+        if (MINIMUM_PCT < 0 || MINIMUM_PCT > 0.5) {
+            errorMsgs.add("MINIMUM_PCT was set to " + MINIMUM_PCT + ". It must be between 0 and 0.5 so all data categories don't get discarded.");
+        }
 
-         return super.customCommandLineValidation();
+        if (!checkRInstallation(Histogram_FILE != null)) {
+            errorMsgs.add("R is not installed on this machine. It is required for creating the chart.");
+        }
+
+        return errorMsgs.isEmpty() ? null : errorMsgs.toArray(new String[errorMsgs.size()]);
     }
 
     @Override protected boolean usesNoRefReads() { return false; }
