@@ -24,18 +24,8 @@
 
 package picard.arrays;
 
-import org.broadinstitute.barclay.help.DocumentedFeature;
-import picard.arrays.illumina.ArraysControlInfo;
-import picard.arrays.illumina.Build37ExtendedIlluminaManifest;
-import picard.arrays.illumina.Build37ExtendedIlluminaManifestRecord;
-import picard.arrays.illumina.IlluminaManifestRecord;
-import picard.arrays.illumina.InfiniumEGTFile;
-import picard.arrays.illumina.InfiniumGTCFile;
-import picard.arrays.illumina.InfiniumGTCRecord;
-import picard.arrays.illumina.InfiniumNormalizationManifest;
-import picard.arrays.illumina.InfiniumVcfFields;
-import picard.pedigree.Sex;
 import htsjdk.samtools.SAMSequenceDictionary;
+import htsjdk.samtools.SAMSequenceRecord;
 import htsjdk.samtools.reference.ReferenceSequenceFile;
 import htsjdk.samtools.reference.ReferenceSequenceFileFactory;
 import htsjdk.samtools.util.IOUtil;
@@ -65,15 +55,22 @@ import org.apache.commons.io.FilenameUtils;
 import org.apache.commons.lang.StringUtils;
 import org.broadinstitute.barclay.argparser.Argument;
 import org.broadinstitute.barclay.argparser.CommandLineProgramProperties;
+import org.broadinstitute.barclay.help.DocumentedFeature;
 import picard.PicardException;
+import picard.arrays.illumina.ArraysControlInfo;
+import picard.arrays.illumina.Build37ExtendedIlluminaManifest;
+import picard.arrays.illumina.Build37ExtendedIlluminaManifestRecord;
+import picard.arrays.illumina.IlluminaManifestRecord;
+import picard.arrays.illumina.InfiniumEGTFile;
+import picard.arrays.illumina.InfiniumGTCFile;
+import picard.arrays.illumina.InfiniumGTCRecord;
+import picard.arrays.illumina.InfiniumVcfFields;
 import picard.cmdline.CommandLineProgram;
 import picard.cmdline.StandardOptionDefinitions;
+import picard.pedigree.Sex;
 
-import java.io.DataInputStream;
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.IOException;
-import java.nio.file.Path;
 import java.text.DecimalFormat;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -87,17 +84,16 @@ import java.util.Map;
 import java.util.Set;
 
 /**
- * Class to convert a GTC file and a BPM file to a VCF file.
+ * Class to convert an Illumina GTC file into a VCF file.
  */
 @CommandLineProgramProperties(
         summary = GtcToVcf.USAGE_DETAILS,
-        oneLineSummary = "Program to convert a GTC file to a VCF",
+        oneLineSummary = "Program to convert an Illumina GTC file to a VCF",
         programGroup = picard.cmdline.programgroups.GenotypingArraysProgramGroup.class
 )
 @DocumentedFeature
 public class GtcToVcf extends CommandLineProgram {
 
-
     static final String USAGE_DETAILS =
             "GtcToVcf takes an Illumina GTC file and converts it to a VCF file using several supporting files. " +
                     "A GTC file is an Illumina-specific file containing called genotypes in AA/AB/BB format. " +
@@ -112,7 +108,7 @@ public class GtcToVcf extends CommandLineProgram {
                     "      OUTPUT=output.vcf \\<br />" +
                     "      EXTENDED_ILLUMINA_MANIFEST=chip_name.extended.csv \\<br />" +
                     "      CLUSTER_FILE=chip_name.egt \\<br />" +
-                    "      ILLUMINA_NORMALIZATION_MANIFEST=chip_name.bpm.csv \\<br />" +
+                    "      ILLUMINA_BEAD_POOL_MANIFEST_FILE=chip_name.bpm \\<br />" +
                     "      SAMPLE_ALIAS=my_sample_alias \\<br />" +
                     "</pre>";
 
@@ -131,8 +127,8 @@ public class GtcToVcf extends CommandLineProgram {
     @Argument(shortName = "CF", doc = "An Illumina cluster file (egt)")
     public File CLUSTER_FILE;
 
-    @Argument(shortName = "NORM_MANIFEST", doc = "An Illumina bead pool manifest (a manifest containing the Illumina normalization ids) (bpm.csv)")
-    public File ILLUMINA_NORMALIZATION_MANIFEST;
+    @Argument(shortName = "BPM_FILE", doc = "The Illumina Bead Pool Manifest (.bpm) file")
+    public File ILLUMINA_BEAD_POOL_MANIFEST_FILE;
 
     @Argument(shortName = "E_GENDER", doc = "The expected gender for this sample.", optional = true)
     public String EXPECTED_GENDER;
@@ -140,6 +136,9 @@ public class GtcToVcf extends CommandLineProgram {
     @Argument(doc = "The sample alias")
     public String SAMPLE_ALIAS;
 
+    @Argument(doc = "The version of the pipeline used to generate this VCF", optional = true)
+    public String PIPELINE_VERSION;
+
     @Argument(doc = "The analysis version of the data used to generate this VCF", optional = true)
     public Integer ANALYSIS_VERSION_NUMBER;
 
@@ -172,11 +171,9 @@ public class GtcToVcf extends CommandLineProgram {
 
     @Override
     protected int doWork() {
-        final InfiniumNormalizationManifest infiniumNormalizationManifest = new InfiniumNormalizationManifest(ILLUMINA_NORMALIZATION_MANIFEST);
-
         Sex fingerprintSex = getFingerprintSex(FINGERPRINT_GENOTYPES_VCF_FILE);
-        String gtcGender = getGenderFromGtcFile(GENDER_GTC, infiniumNormalizationManifest);
-        try (InfiniumGTCFile infiniumGTCFile = new InfiniumGTCFile(new DataInputStream(new FileInputStream(INPUT)), infiniumNormalizationManifest);
+        String gtcGender = getGenderFromGtcFile(GENDER_GTC, ILLUMINA_BEAD_POOL_MANIFEST_FILE);
+        try (InfiniumGTCFile infiniumGTCFile = new InfiniumGTCFile(INPUT, ILLUMINA_BEAD_POOL_MANIFEST_FILE);
              InfiniumEGTFile infiniumEGTFile = new InfiniumEGTFile(CLUSTER_FILE)) {
             final Build37ExtendedIlluminaManifest manifest = setupAndGetManifest(infiniumGTCFile);
 
@@ -190,14 +187,16 @@ public class GtcToVcf extends CommandLineProgram {
                             VariantContext.class,
                             new VCFRecordCodec(vcfHeader),
                             new VariantContextComparator(refSeq.getSequenceDictionary()),
-                            MAX_RECORDS_IN_RAM,
-                            TMP_DIR.stream().map(File::toPath).toArray(Path[]::new));
+                            MAX_RECORDS_IN_RAM);
 
             // fill the sorting collection
             fillContexts(contexts, infiniumGTCFile, manifest, infiniumEGTFile);
 
             writeVcf(contexts, OUTPUT, refSeq.getSequenceDictionary(), vcfHeader);
 
+            // not sure this is needed but it's definitely **cleaner**
+            contexts.cleanup();
+
             return 0;
         } catch (IOException e) {
             throw new PicardException("Error processing GTC File: " + INPUT.getAbsolutePath(), e);
@@ -209,10 +208,14 @@ public class GtcToVcf extends CommandLineProgram {
 
         IOUtil.assertFileIsReadable(INPUT);
         IOUtil.assertFileIsReadable(EXTENDED_ILLUMINA_MANIFEST);
+        IOUtil.assertFileIsReadable(ILLUMINA_BEAD_POOL_MANIFEST_FILE);
         IOUtil.assertFileIsWritable(OUTPUT);
         refSeq = ReferenceSequenceFileFactory.getReferenceSequenceFile(REFERENCE_SEQUENCE);
         final SAMSequenceDictionary sequenceDictionary = refSeq.getSequenceDictionary();
         final String assembly = sequenceDictionary.getSequence(0).getAssembly();
+        if (assembly == null) {
+            return new String[]{"Assembly tag ('" + SAMSequenceRecord.ASSEMBLY_TAG + "') is required in the sequence dictionary)."};
+        }
         if (!assembly.equals("GRCh37")) {
             return new String[]{"The selected reference sequence ('" + assembly + "') is not supported.  This tool is currently only implemented to support NCBI Build 37 / HG19 Reference Sequence."};
         }
@@ -263,10 +266,10 @@ public class GtcToVcf extends CommandLineProgram {
         return Sex.Unknown;
     }
 
-    private String getGenderFromGtcFile(final File gtcFile, final InfiniumNormalizationManifest infiniumNormalizationManifest) {
+    private String getGenderFromGtcFile(final File gtcFile, final File bpmFile) {
         String gtcGender = null;
         if (gtcFile != null) {
-            try (InfiniumGTCFile infiniumGTCGenderFile = new InfiniumGTCFile(new DataInputStream(new FileInputStream(gtcFile)), infiniumNormalizationManifest)) {
+            try (InfiniumGTCFile infiniumGTCGenderFile = new InfiniumGTCFile(gtcFile, bpmFile)) {
                 gtcGender = infiniumGTCGenderFile.getGender();
             } catch (IOException e) {
                 throw new PicardException("Error processing GTC File: " + gtcFile.getAbsolutePath(), e);
@@ -538,6 +541,9 @@ public class GtcToVcf extends CommandLineProgram {
         if (EXPECTED_GENDER != null) {
             lines.add(new VCFHeaderLine(InfiniumVcfFields.EXPECTED_GENDER, EXPECTED_GENDER));
         }
+        if (PIPELINE_VERSION != null) {
+            lines.add(new VCFHeaderLine(InfiniumVcfFields.PIPELINE_VERSION, PIPELINE_VERSION));
+        }
         //add control codes
         final int measurementCount = gtcFile.getRawControlXIntensities().length / ArraysControlInfo.CONTROL_INFO.length;
         for (int i = 0; i < ArraysControlInfo.CONTROL_INFO.length; i++) {
@@ -548,6 +554,7 @@ public class GtcToVcf extends CommandLineProgram {
             lines.add(new VCFHeaderLine(controlInfo.getControl(), controlInfo.toString() + "|" + redIntensity + "|" + greenIntensity));
         }
         lines.add(new VCFHeaderLine(InfiniumVcfFields.FINGERPRINT_GENDER, fingerprintGender.name()));
+        lines.add(new VCFHeaderLine(InfiniumVcfFields.GTC_CALL_RATE, String.valueOf(gtcFile.getCallRate())));
         if (gtcGender != null) {
             lines.add(new VCFHeaderLine(InfiniumVcfFields.AUTOCALL_GENDER, gtcGender));
         } else {
@@ -614,4 +621,4 @@ public class GtcToVcf extends CommandLineProgram {
         header.setSequenceDictionary(dict);
         return header;
     }
-}
\ No newline at end of file
+}
