@@ -23,7 +23,6 @@
  */
 package picard.illumina;
 
-import com.sun.xml.internal.rngom.parse.host.Base;
 import htsjdk.samtools.SAMRecordQueryNameComparator;
 import htsjdk.samtools.SAMUtils;
 import htsjdk.samtools.fastq.BasicFastqWriter;
@@ -388,10 +387,6 @@ public class IlluminaBasecallsToFastq extends CommandLineProgram {
         return new FastqRecordsWriter(templateWriters, sampleBarcodeWriters, molecularBarcodeWriters);
     }
 
-    public static void main(final String[] args) {
-        new IlluminaBasecallsToFastq().instanceMainWithExit(args);
-    }
-
     /**
      * Container for various FastqWriters, one for each template read, one for each sample barcode read,
      * and one for each molecular barcode read.
