@@ -33,6 +33,7 @@ import htsjdk.samtools.SamInputResource;
 import htsjdk.samtools.SamReader;
 import htsjdk.samtools.SamReaderFactory;
 import htsjdk.samtools.util.CloserUtil;
+import htsjdk.samtools.util.FileExtensions;
 import htsjdk.samtools.util.IOUtil;
 import htsjdk.samtools.util.Log;
 import org.broadinstitute.barclay.argparser.Argument;
@@ -105,13 +106,14 @@ public class BuildBamIndex extends CommandLineProgram {
                 baseFileName = inputFile.getAbsolutePath();
             }
 
-            if (baseFileName.endsWith(BamFileIoUtils.BAM_FILE_EXTENSION)) {
+            // only BAI indices can be created for now, although CSI indices can be read as well
+            if (baseFileName.endsWith(FileExtensions.BAM)) {
 
                 final int index = baseFileName.lastIndexOf('.');
-                OUTPUT = new File(baseFileName.substring(0, index) + BAMIndex.BAMIndexSuffix);
+                OUTPUT = new File(baseFileName.substring(0, index) + FileExtensions.BAI_INDEX);
 
             } else {
-                OUTPUT = new File(baseFileName + BAMIndex.BAMIndexSuffix);
+                OUTPUT = new File(baseFileName + FileExtensions.BAI_INDEX);
             }
         }
 
@@ -132,7 +134,7 @@ public class BuildBamIndex extends CommandLineProgram {
                     .open(inputFile);
         }
 
-        if (bam.type() != SamReader.Type.BAM_TYPE) {
+        if (bam.type() != SamReader.Type.BAM_TYPE && bam.type() != SamReader.Type.BAM_CSI_TYPE) {
             throw new SAMException("Input file must be bam file, not sam file.");
         }
 
