@@ -103,7 +103,11 @@ public class DuplicationMetrics extends MergeableMetricBase {
         this.ESTIMATED_LIBRARY_SIZE = estimateLibrarySize(this.READ_PAIRS_EXAMINED - this.READ_PAIR_OPTICAL_DUPLICATES,
                 this.READ_PAIRS_EXAMINED - this.READ_PAIR_DUPLICATES);
 
-        PERCENT_DUPLICATION = (UNPAIRED_READ_DUPLICATES + READ_PAIR_DUPLICATES * 2) / (double) (UNPAIRED_READS_EXAMINED + READ_PAIRS_EXAMINED * 2);
+        if (UNPAIRED_READS_EXAMINED + READ_PAIRS_EXAMINED != 0) {
+            PERCENT_DUPLICATION = (UNPAIRED_READ_DUPLICATES + READ_PAIR_DUPLICATES * 2) / (double) (UNPAIRED_READS_EXAMINED + READ_PAIRS_EXAMINED * 2);
+        } else {
+            PERCENT_DUPLICATION = (double) 0;
+        }
     }
 
     /**
@@ -209,7 +213,7 @@ public class DuplicationMetrics extends MergeableMetricBase {
         for (double x = 1; x <= 100; x += 1) {
             histo.increment(x, estimateRoi(ESTIMATED_LIBRARY_SIZE, x, READ_PAIRS_EXAMINED, uniquePairs));
         }
-
+        histo.setValueLabel("CoverageMult");
         return histo;
     }
 
