@@ -104,7 +104,7 @@ import java.util.List;
  *      F2=reverse_reads.fastq \
  *      O=unaligned_read_pairs.bam \
  *      SM=sample001 \
- *      RG=rg0013 
+ *      RG=rg0013
  * </pre>
  */
 @CommandLineProgramProperties(
@@ -113,40 +113,40 @@ import java.util.List;
         programGroup = ReadDataManipulationProgramGroup.class)
 @DocumentedFeature
 public class FastqToSam extends CommandLineProgram {
-    static final String USAGE_SUMMARY = 
-    		"Converts a FASTQ file to an unaligned BAM or SAM file";
-    static final String USAGE_DETAILS = 
-    		"<p>Output read records will contain the original base calls and quality scores will be " +
-    		"translated depending on the base quality score encoding: FastqSanger, FastqSolexa and FastqIllumina.</p>" +
+    static final String USAGE_SUMMARY =
+            "Converts a FASTQ file to an unaligned BAM or SAM file";
+    static final String USAGE_DETAILS =
+            "<p>Output read records will contain the original base calls and quality scores will be " +
+                    "translated depending on the base quality score encoding: FastqSanger, FastqSolexa and FastqIllumina.</p>" +
         "<p>There are also arguments to provide values for SAM header and read attributes that are not present in FASTQ " +
-        "(e.g see RG or SM below).</p>" + 
+        "(e.g see RG or SM below).</p>" +
         "<h3>Inputs</h3>" +
         "<p>One FASTQ file name for single-end or two for pair-end sequencing input data. " +
         "These files might be in gzip compressed format (when file name is ending with \".gz\").</p>" +
-        "<p>Alternatively, for larger inputs you can provide a collection of FASTQ files indexed by their name " + 
+        "<p>Alternatively, for larger inputs you can provide a collection of FASTQ files indexed by their name " +
         "(see USE_SEQUENCIAL_FASTQ for details below).</p>" +
         "<p>By default, this tool will try to guess the base quality score encoding. However you can indicate it explicitly " +
         "using the QUALITY_FORMAT argument.</p>" +
         "<h3>Output</h3>" +
         "<p>A single unaligned BAM or SAM file. By default, the records are sorted by query (read) name.</p>" +
         "<h3>Usage examples</h3>" +
-        "<h4>Example 1:</h4>" + 
-        "<p>Single-end sequencing FASTQ file conversion. All reads are annotated " + 
-        "as belonging to the \"rg0013\" read group that in turn is part of the sample \"sample001\".</p>" + 
+        "<h4>Example 1:</h4>" +
+        "<p>Single-end sequencing FASTQ file conversion. All reads are annotated " +
+        "as belonging to the \"rg0013\" read group that in turn is part of the sample \"sample001\".</p>" +
         "<pre>java -jar picard.jar FastqToSam \\\n" +
         "        F1=input_reads.fastq \\\n" +
-        "        O=unaligned_reads.bam \\\n" + 
-        "        SM=sample001 \\\n" + 
+        "        O=unaligned_reads.bam \\\n" +
+        "        SM=sample001 \\\n" +
         "        RG=rg0013</pre>" +
-        "<h4>Example 2:</h4>" + 
-        "<p>Similar to example 1 above, but for paired-end sequencing.</p>" + 
+        "<h4>Example 2:</h4>" +
+        "<p>Similar to example 1 above, but for paired-end sequencing.</p>" +
         "<pre>java -jar picard.jar FastqToSam \\\n" +
-        "       F1=forward_reads.fastq \\\n" + 
+        "       F1=forward_reads.fastq \\\n" +
         "       F2=reverse_reads.fastq \\\n" +
-        "       O=unaligned_read_pairs.bam \\\n" + 
-        "       SM=sample001 \\\n" + 
+        "       O=unaligned_read_pairs.bam \\\n" +
+        "       SM=sample001 \\\n" +
         "       RG=rg0013</pre><hr />";
-    
+
     private static final Log LOG = Log.getInstance(FastqToSam.class);
 
     @Argument(shortName="F1", doc="Input fastq file (optionally gzipped) for single end data, or first read in paired end data.")
@@ -154,8 +154,18 @@ public class FastqToSam extends CommandLineProgram {
 
     @Argument(shortName="F2", doc="Input fastq file (optionally gzipped) for the second read of paired end data.", optional=true)
     public File FASTQ2;
-    
-    @Argument(doc="Use sequential fastq files with the suffix <prefix>_###.fastq or <prefix>_###.fastq.gz", optional=true)
+
+    @Argument(doc="Use sequential fastq files with the suffix <prefix>_###.fastq or <prefix>_###.fastq.gz." +
+            "The files should be named:\n" +
+            "    <prefix>_001.<extension>, <prefix>_002.<extension>, ..., <prefix>_XYZ.<extension>\n" +
+            " The base files should be:\n" +
+            "    <prefix>_001.<extension>\n" +
+            " An example would be:\n" +
+            "    RUNNAME_S8_L005_R1_001.fastq\n" +
+            "    RUNNAME_S8_L005_R1_002.fastq\n" +
+            "    RUNNAME_S8_L005_R1_003.fastq\n" +
+            "    RUNNAME_S8_L005_R1_004.fastq\n" +
+            "RUNNAME_S8_L005_R1_001.fastq should be provided as FASTQ.", optional=true)
     public boolean USE_SEQUENTIAL_FASTQS = false;
 
     @Argument(shortName="V", doc="A value describing how the quality values are encoded in the input FASTQ file.  " +
@@ -178,7 +188,7 @@ public class FastqToSam extends CommandLineProgram {
     @Argument(shortName="PU", doc="The platform unit (often run_barcode.lane) to insert into the read group header", optional=true)
     public String PLATFORM_UNIT;
 
-    @Argument(shortName="PL", doc="The platform type (e.g. illumina, solid) to insert into the read group header", optional=true)
+    @Argument(shortName="PL", doc="The platform type (e.g. ILLUMINA, SOLID) to insert into the read group header", optional=true)
     public String PLATFORM;
 
     @Argument(shortName="CN", doc="The sequencing center from which the data originated", optional=true)
@@ -194,7 +204,7 @@ public class FastqToSam extends CommandLineProgram {
     public String PLATFORM_MODEL;
 
     @Argument(doc="Comment(s) to include in the merged output file's header.", optional=true, shortName="CO")
-    public List<String> COMMENT = new ArrayList<String>();
+    public List<String> COMMENT = new ArrayList<>();
 
     @Argument(shortName = "DS", doc = "Inserted into the read group header", optional = true)
     public String DESCRIPTION;
@@ -250,10 +260,6 @@ public class FastqToSam extends CommandLineProgram {
         return qualityFormat;
     }
 
-    /** Stock main method. */
-    public static void main(final String[] argv) {
-        System.exit(new FastqToSam().instanceMain(argv));
-    }
 
     /**
      * Get a list of FASTQs that are sequentially numbered based on the first (base) fastq.
@@ -269,7 +275,7 @@ public class FastqToSam extends CommandLineProgram {
      * where `baseFastq` is the first in that list.
      */
     protected static List<File> getSequentialFileList(final File baseFastq) {
-        final List<File> files = new ArrayList<File>();
+        final List<File> files = new ArrayList<>();
         files.add(baseFastq);
 
         // Find the correct extension used in the base FASTQ
@@ -285,7 +291,7 @@ public class FastqToSam extends CommandLineProgram {
         if (null == fastqExtensions) {
             throw new PicardException(String.format("Could not parse the FASTQ extension (expected '_001' + '%s'): %s", FastqExtensions.values().toString(), baseFastq));
         }
-        
+
         // Find all the files
         for (int idx = 2; true; idx++) {
             String fastq = baseFastq.getAbsolutePath();
@@ -297,7 +303,7 @@ public class FastqToSam extends CommandLineProgram {
             }
             files.add(new File(fastq));
         }
-        
+
         return files;
     }
 
@@ -316,10 +322,10 @@ public class FastqToSam extends CommandLineProgram {
         QUALITY_FORMAT = FastqToSam.determineQualityFormat(fileToFastqReader(FASTQ),
                 (FASTQ2 == null) ? null : fileToFastqReader(FASTQ2),
                 QUALITY_FORMAT);
-        
+
         // Lists for sequential files, but also used when not sequential
-        final List<FastqReader> readers1 = new ArrayList<FastqReader>();
-        final List<FastqReader> readers2 = new ArrayList<FastqReader>();
+        final List<FastqReader> readers1 = new ArrayList<>();
+        final List<FastqReader> readers2 = new ArrayList<>();
 
         if (USE_SEQUENTIAL_FASTQS) {
             // Get all the files
@@ -495,7 +501,7 @@ public class FastqToSam extends CommandLineProgram {
      * <li> Paired reads must either have the exact same read names or they must contain at least one "/"
      * <li> and the First pair read name must end with "/1" and second pair read name ends with "/2"
      * <li> The baseName (read name part before the /) must be the same for both read names
-     * <li> If the read names are exactly the same but end in "/2" or "/1" then an exception will be thrown 
+     * <li> If the read names are exactly the same but end in "/2" or "/1" then an exception will be thrown
      * </ul>
      */
     String getBaseName(final String readName1, final String readName2, final FastqReader freader1, final FastqReader freader2) {
@@ -515,7 +521,7 @@ public class FastqToSam extends CommandLineProgram {
         final boolean num2Blank = StringUtil.isBlank(num2);
         if (num1Blank || num2Blank) {
             if(!num1Blank) throw new PicardException(error(freader1,"Pair 1 number is missing (" +readName1+ "). Both pair numbers must be present or neither."));       //num1 != blank and num2   == blank
-            else if(!num2Blank) throw new PicardException(error(freader2, "Pair 2 number is missing (" +readName2+ "). Both pair numbers must be present or neither.")); //num1 == blank and num =2 != blank 
+            else if(!num2Blank) throw new PicardException(error(freader2, "Pair 2 number is missing (" +readName2+ "). Both pair numbers must be present or neither.")); //num1 == blank and num =2 != blank
         } else {
             if (!num1.equals("1")) throw new PicardException(error(freader1,"Pair 1 number must be 1 ("+readName1+")"));
             if (!num2.equals("2")) throw new PicardException(error(freader2,"Pair 2 number must be 2 ("+readName2+")"));
@@ -536,7 +542,7 @@ public class FastqToSam extends CommandLineProgram {
             result[1] = null;
         } else {
             result[1] = readName.substring(idx+1, readName.length()); // should be a 1 or 2
-            
+
             if(!result[1].equals("1") && !result[1].equals("2")) {    //if not a 1 or 2 then names must be identical
                 result[0] = readName;
                 result[1] = null;
