@@ -239,13 +239,6 @@ public class MarkDuplicates extends AbstractMarkDuplicatesCommandLineProgram {
         DUPLICATE_SCORING_STRATEGY = ScoringStrategy.SUM_OF_BASE_QUALITIES;
     }
 
-    /**
-     * Stock main method.
-     */
-    public static void main(final String[] args) {
-        new MarkDuplicates().instanceMainWithExit(args);
-    }
-
     /**
      * Main work method.  Reads the BAM file once and collects sorted information about
      * the 5' ends of both ends of each read (or just one end in the case of pairs).
@@ -332,24 +325,7 @@ public class MarkDuplicates extends AbstractMarkDuplicatesCommandLineProgram {
             while (iterator.hasNext()) {
                 final SAMRecord rec = iterator.next();
 
-                final String library = LibraryIdGenerator.getLibraryName(header, rec);
-                DuplicationMetrics metrics = libraryIdGenerator.getMetricsByLibrary(library);
-                if (metrics == null) {
-                    metrics = new DuplicationMetrics();
-                    metrics.LIBRARY = library;
-                    libraryIdGenerator.addMetricsByLibrary(library, metrics);
-                }
-
-                // First bring the simple metrics up to date
-                if (rec.getReadUnmappedFlag()) {
-                    ++metrics.UNMAPPED_READS;
-                } else if (rec.isSecondaryOrSupplementary()) {
-                    ++metrics.SECONDARY_OR_SUPPLEMENTARY_RDS;
-                } else if (!rec.getReadPairedFlag() || rec.getMateUnmappedFlag()) {
-                    ++metrics.UNPAIRED_READS_EXAMINED;
-                } else {
-                    ++metrics.READ_PAIRS_EXAMINED; // will need to be divided by 2 at the end
-                }
+                DuplicationMetrics metrics = AbstractMarkDuplicatesCommandLineProgram.addReadToLibraryMetrics(rec, header, libraryIdGenerator);
 
                 // Now try and figure out the next duplicate index (if going by coordinate. if going by query name, only do this
                 // if the query name has changed.
@@ -362,15 +338,7 @@ public class MarkDuplicates extends AbstractMarkDuplicatesCommandLineProgram {
                 if (isDuplicate) {
                     rec.setDuplicateReadFlag(true);
 
-                    // only update duplicate counts for "decider" reads, not tag-a-long reads
-                    if (!rec.isSecondaryOrSupplementary() && !rec.getReadUnmappedFlag()) {
-                        // Update the duplication metrics
-                        if (!rec.getReadPairedFlag() || rec.getMateUnmappedFlag()) {
-                            ++metrics.UNPAIRED_READ_DUPLICATES;
-                        } else {
-                            ++metrics.READ_PAIR_DUPLICATES;// will need to be divided by 2 at the end
-                        }
-                    }
+                    AbstractMarkDuplicatesCommandLineProgram.addDuplicateReadToMetrics(rec, metrics);
                 } else {
                     rec.setDuplicateReadFlag(false);
                 }
@@ -459,19 +427,24 @@ public class MarkDuplicates extends AbstractMarkDuplicatesCommandLineProgram {
             }
 
             // remember to close the inputs
+            log.info("Writing complete. Closing input iterator.");
             iterator.close();
 
+            log.info("Duplicate Index cleanup.");
             this.duplicateIndexes.cleanup();
             if (TAG_DUPLICATE_SET_MEMBERS) {
+                log.info("Representative read Index cleanup.");
                 this.representativeReadIndicesForDuplicates.cleanup();
             }
 
+            log.info("Getting Memory Stats.");
             reportMemoryStats("Before output close");
         }
+        log.info("Closed outputs. Getting more Memory Stats.");
         reportMemoryStats("After output close");
 
         // Write out the metrics
-        finalizeAndWriteMetrics(libraryIdGenerator);
+        finalizeAndWriteMetrics(libraryIdGenerator, getMetricsFile(), METRICS_FILE);
 
         return 0;
     }
@@ -535,7 +508,8 @@ public class MarkDuplicates extends AbstractMarkDuplicatesCommandLineProgram {
         final SamHeaderAndIterator headerAndIterator = openInputs(true);
         final SAMFileHeader.SortOrder assumedSortOrder = headerAndIterator.header.getSortOrder();
         final SAMFileHeader header = headerAndIterator.header;
-        final ReadEndsForMarkDuplicatesMap tmp = new DiskBasedReadEndsForMarkDuplicatesMap(MAX_FILE_HANDLES_FOR_READ_ENDS_MAP, diskCodec);
+        final ReadEndsForMarkDuplicatesMap tmp = assumedSortOrder == SAMFileHeader.SortOrder.queryname ?
+                new MemoryBasedReadEndsForMarkDuplicatesMap() :  new DiskBasedReadEndsForMarkDuplicatesMap(MAX_FILE_HANDLES_FOR_READ_ENDS_MAP, diskCodec);
         long index = 0;
         final ProgressLogger progress = new ProgressLogger(log, (int) 1e6, "Read");
         final CloseableIterator<SAMRecord> iterator = headerAndIterator.iterator;
@@ -913,7 +887,7 @@ public class MarkDuplicates extends AbstractMarkDuplicatesCommandLineProgram {
                     // We expect end.read2IndexInFile==read1IndexInFile when we are in queryname sorted files, as the read-pairs
                     // will be sorted together and nextIndexIfNeeded() will only pull one index from opticalDuplicateIndexes.
                     // This means that in queryname sorted order we will only pull from the sorting collection once,
-                    // where as we would pull twice for coordinate sorted files. 
+                    // where as we would pull twice for coordinate sorted files.
                     if (end.read2IndexInFile != end.read1IndexInFile) {
                         this.opticalDuplicateIndexes.add(end.read2IndexInFile);
                     }
@@ -1036,8 +1010,8 @@ public class MarkDuplicates extends AbstractMarkDuplicatesCommandLineProgram {
                 compareDifference = lhs.getY() - rhs.getY();
             }
 
-            // The following is arbitrary (especially given the dependence above on the hash) and is only
-            // there for completeness. Other implementations may chose to forgo this tiebreak if they do have
+            // The following is arbitrary and is only included for completeness.
+            // Other implementations may chose to forgo this tiebreak if they do not have
             // access to the index-in-file of the records (e.g. SPARK implmentations)
 
             if (compareDifference == 0) {
