@@ -74,19 +74,12 @@ import java.util.Set;
 public class SimpleMarkDuplicatesWithMateCigar extends MarkDuplicates {
     private final Log log = Log.getInstance(MarkDuplicatesWithMateCigar.class);
 
-    /** Stock main method. */
-    public static void main(final String[] args) {
-        new MarkDuplicatesWithMateCigar().instanceMainWithExit(args);
-    }
-
-    private class ReadEndsForSimpleMarkDuplicatesWithMateCigar extends ReadEnds {
-    }
+    private class ReadEndsForSimpleMarkDuplicatesWithMateCigar extends ReadEnds {}
     
     private static boolean isPairedAndBothMapped(final SAMRecord record) {
         return record.getReadPairedFlag() &&
                 !record.getReadUnmappedFlag() &&
                 !record.getMateUnmappedFlag();
-        
     }
 
     /**
@@ -227,7 +220,7 @@ public class SimpleMarkDuplicatesWithMateCigar extends MarkDuplicates {
         log.info("Marking " + numDuplicates + " records as duplicates.");
 
         // Write out the metrics
-        finalizeAndWriteMetrics(libraryIdGenerator);
+        finalizeAndWriteMetrics(libraryIdGenerator, getMetricsFile(), METRICS_FILE);
 
         return 0;
     }
