@@ -34,6 +34,7 @@ import picard.sam.DuplicationMetrics;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Optional;
+import java.util.TreeMap;
 
 /**
  * A class to generate library Ids and keep duplication metrics by library IDs.
@@ -47,7 +48,7 @@ public class LibraryIdGenerator {
     private final SAMFileHeader header;
     private final Map<String, Short> libraryIds = new HashMap<String, Short>(); // from library string to library id
     private short nextLibraryId = 1;
-    private final Map<String, DuplicationMetrics> metricsByLibrary = new HashMap<>();
+    private final Map<String, DuplicationMetrics> metricsByLibrary = new TreeMap<>();
     private final Histogram<Short> opticalDuplicatesByLibraryId = new Histogram<>();
     private final Histogram<Double> duplicateCountHist = new Histogram<>("set_size", "all_sets");
     private final Histogram<Double> nonOpticalDuplicateCountHist = new Histogram<>("set_size", "non_optical_sets");
@@ -92,9 +93,9 @@ public class LibraryIdGenerator {
     }
 
     public static String getReadGroupLibraryName(final SAMReadGroupRecord readGroup) {
-		return Optional.ofNullable(readGroup.getLibrary())
-				.orElse(UNKNOWN_LIBRARY);
-	}
+        return Optional.ofNullable(readGroup.getLibrary())
+                .orElse(UNKNOWN_LIBRARY);
+    }
    
     /**
      * Gets the library name from the header for the record. If the RG tag is not present on
