@@ -25,7 +25,9 @@
 package picard.analysis;
 
 import htsjdk.samtools.metrics.MetricsFile;
+import htsjdk.samtools.util.Histogram;
 import org.testng.Assert;
+import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 import picard.cmdline.CommandLineProgramTest;
 import picard.util.TestNGUtil;
@@ -34,6 +36,8 @@ import java.io.File;
 import java.io.FileReader;
 import java.io.IOException;
 import java.text.NumberFormat;
+import java.util.ArrayList;
+import java.util.List;
 
 /**
  * Tests CollectAlignmentSummaryStatistics
@@ -41,7 +45,7 @@ import java.text.NumberFormat;
  * @author Doug Voet (dvoet at broadinstitute dot org)
  */
 public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
-    private static final File TEST_DATA_DIR = new File("testdata/picard/sam");
+    public static final File TEST_DATA_DIR = new File("testdata/picard/sam/AlignmentSummaryMetrics");
 
     public String getCommandLineProgramName() {
         return CollectAlignmentSummaryMetrics.class.getSimpleName();
@@ -51,8 +55,7 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
     public void test() throws IOException {
         final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test.sam");
         final File reference = new File(TEST_DATA_DIR, "summary_alignment_stats_test.fasta");
-        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
-        outfile.deleteOnExit();
+        final File outfile = getTempOutputFile("test", ".txt");
         final String[] args = new String[]{
                 "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
@@ -123,8 +126,8 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
     public void testBisulfite() throws IOException {
         final File input = new File(TEST_DATA_DIR, "summary_alignment_bisulfite_test.sam");
         final File reference = new File(TEST_DATA_DIR, "summary_alignment_stats_test.fasta");
-        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
-        outfile.deleteOnExit();
+        final File outfile = getTempOutputFile("testBisulfite", ".txt");
+
         final String[] args = new String[]{
                 "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
@@ -187,8 +190,8 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
     public void testBisulfiteButNot() throws IOException {
         final File input = new File(TEST_DATA_DIR, "summary_alignment_bisulfite_test.sam");
         final File reference = new File(TEST_DATA_DIR, "summary_alignment_stats_test.fasta");
-        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
-        outfile.deleteOnExit();
+        final File outfile = getTempOutputFile("testBisulfiteButNot", ".txt");
+
         final String[] args = new String[]{
                 "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
@@ -250,8 +253,8 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
     @Test
     public void testNoReference() throws IOException {
         final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test.sam");
-        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
-        outfile.deleteOnExit();
+        final File outfile = getTempOutputFile("testNoReference", ".txt");
+
         final String[] args = new String[]{
                 "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
@@ -310,8 +313,8 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
     @Test
     public void testZeroLengthReads() throws IOException {
         final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test2.sam");
-        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
-        outfile.deleteOnExit();
+        final File outfile = getTempOutputFile("testZeroLengthReads", ".txt");
+
         final String[] args = new String[]{
                 "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
@@ -331,8 +334,8 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
     @Test
     public void testMultipleLevelsOfMetrics() throws IOException {
         final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test_multiple.sam");
-        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
-        outfile.deleteOnExit();
+        final File outfile = getTempOutputFile("testMultipleLevelsOfMetrics", ".txt");
+
         final String[] args = new String[]{
                 "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
@@ -588,8 +591,8 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
     public void testChimeras() throws IOException {
         final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test_chimeras.sam");
         final File reference = new File(TEST_DATA_DIR, "summary_alignment_stats_test.fasta");
-        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
-        outfile.deleteOnExit();
+        final File outfile = getTempOutputFile("testChimeras", ".txt");
+
         final String[] args = new String[]{
                 "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
@@ -616,8 +619,8 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
     @Test
     public void testAdapterReads() throws IOException {
         final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test_adapter_reads.sam");
-        final File outfile = File.createTempFile("alignmentMetrics", ".txt");
-        outfile.deleteOnExit();
+        final File outfile = getTempOutputFile("testAdapterReads", ".txt");
+
         final String[] args = new String[]{
                 "INPUT=" + input.getAbsolutePath(),
                 "OUTPUT=" + outfile.getAbsolutePath(),
@@ -642,4 +645,121 @@ public class CollectAlignmentSummaryMetricsTest extends CommandLineProgramTest {
             }
         }
     }
+
+
+    @DataProvider
+    Object[][] fileForTestReadLengthHistogram(){
+        return new Object[][]{
+                new Object[]{"summary_alignment_stats_test.sam"},
+                new Object[]{"summary_alignment_stats_test2.sam"},
+                new Object[]{"summary_alignment_stats_test3.sam"}
+        };
+    }
+
+    @Test(dataProvider = "fileForTestReadLengthHistogram")
+    public void testReadLengthHistogram(final String fileToUse) throws IOException {
+        final File input = new File(TEST_DATA_DIR, fileToUse);
+        final File outFile = getTempOutputFile("testReadLengthHistogram", ".txt");
+
+        final List<String> argsList = new ArrayList<>();
+        final File outHist = getTempOutputFile("testReadLengthHistogram", ".pdf");
+
+        argsList.add("INPUT=" + input.getAbsolutePath());
+        argsList.add("OUTPUT=" + outFile.getAbsolutePath());
+        argsList.add("HISTOGRAM_FILE=" + outHist);
+
+        Assert.assertEquals(runPicardCommandLine(argsList.toArray(new String[0])),0);
+
+        Assert.assertTrue(outHist.exists());
+    }
+
+
+    @DataProvider()
+    Object[][] TrueFalse() {
+        return new Object[][]{
+                {true},
+                {false},
+        };
+    }
+
+    @Test(dataProvider = "TrueFalse")
+    public void testReadLengthHistogram(final boolean plotChart) throws IOException {
+        final File input = new File(TEST_DATA_DIR, "summary_alignment_stats_test3.sam");
+        final File outFile = getTempOutputFile("testReadLengthHistogram", ".txt");
+
+        final List<String> argsList = new ArrayList<>();
+        argsList.add("INPUT=" + input.getAbsolutePath());
+        argsList.add("OUTPUT=" + outFile.getAbsolutePath());
+        final File outHist;
+
+        if (plotChart) {
+            outHist = getTempOutputFile("testReadLengthHistogram", ".pdf");
+            argsList.add("HISTOGRAM_FILE=" + outHist);
+        } else {
+            outHist = null;
+        }
+
+        final String[] args = argsList.toArray(new String[0]);
+
+        Assert.assertEquals(runPicardCommandLine(args), 0);
+
+        if (plotChart) {
+            Assert.assertTrue(outHist.exists());
+        }
+
+        final MetricsFile<AlignmentSummaryMetrics, Integer> output = new MetricsFile<>();
+        try (FileReader reader = new FileReader(outFile)) {
+            output.read(reader);
+        }
+
+        Assert.assertFalse(output.getMetrics().isEmpty());
+
+
+        for (final AlignmentSummaryMetrics metrics : output.getMetrics()) {
+            if (metrics.CATEGORY != AlignmentSummaryMetrics.Category.SECOND_OF_PAIR) {
+                Assert.assertEquals(metrics.PCT_HARDCLIP, 10 / (double) 66, 0.0001);
+                Assert.assertEquals(metrics.PCT_SOFTCLIP, 23 / (double) 66, 0.0001);
+                Assert.assertEquals(metrics.AVG_POS_3PRIME_SOFTCLIP_LENGTH, 22 / (double) 2, 0.0001);
+            } else {
+                Assert.assertEquals(metrics.PCT_HARDCLIP, 0D);
+                Assert.assertEquals(metrics.PCT_SOFTCLIP, 0D);
+                Assert.assertEquals(metrics.AVG_POS_3PRIME_SOFTCLIP_LENGTH, 0D);
+            }
+        }
+
+        Assert.assertFalse(output.getAllHistograms().isEmpty());
+
+        for (final Histogram<Integer> histogram : output.getAllHistograms()) {
+            switch (histogram.getValueLabel()) {
+                case "PAIRED_TOTAL_LENGTH_COUNT":
+                case "UNPAIRED_TOTAL_LENGTH_COUNT":
+                    Assert.assertEquals(histogram.getSum(), 66D); //1+2+11+10+42
+                    break;
+                case "PAIRED_ALIGNED_LENGTH_COUNT":
+                case "UNPAIRED_ALIGNED_LENGTH_COUNT":
+                    Assert.assertEquals(histogram.getSum(), 43D); //1+2+10+10+10+10;
+                    break;
+            }
+
+            Assert.assertFalse(histogram.isEmpty());
+            if (histogram.getValueLabel().equals("PAIRED_TOTAL_LENGTH_COUNT")) {
+                for (int i = 0; i < histogram.getMax(); i++) {
+                    switch (i) {
+                        case 1:
+                        case 2:
+                        case 10:
+                        case 11:
+                        case 42:
+                            Assert.assertEquals(histogram.get(i).getValue(), 1D);
+                            break;
+                        default:
+                            Assert.assertTrue(
+                                    histogram.get(i) == null ||
+                                            histogram.get(i).getValue() == 0D);
+                            break;
+                    }
+                }
+            }
+        }
+    }
 }
