@@ -41,11 +41,7 @@ import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 
 import java.io.File;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
+import java.util.*;
 
 /**
  * This class defines the individual test cases to run. The actual running of the test is done
@@ -69,6 +65,7 @@ public class MarkDuplicatesTest extends AbstractMarkDuplicatesCommandLineProgram
     @Test
     public void testTwoMappedPairsWithSoftClippingFirstOfPairOnly() {
         final AbstractMarkDuplicatesCommandLineProgramTester tester = getTester();
+        tester.getSamRecordSetBuilder().setReadLength(76);
         // NB: no duplicates
         // 5'1: 2, 5'2:46+73M=118
         // 5'1: 2, 5'2:51+68M=118
@@ -93,13 +90,14 @@ public class MarkDuplicatesTest extends AbstractMarkDuplicatesCommandLineProgram
             // Run MarkDuplicates, merging the 3 input files, and either enabling or suppressing PG header
             // record creation according to suppressPg.
             final MarkDuplicates markDuplicates = new MarkDuplicates();
-            final ArrayList<String> args = new ArrayList<String>();
+            final ArrayList<String> args = new ArrayList<>();
             for (int i = 1; i <= 3; ++i) {
                 args.add("INPUT=" + new File(TEST_DATA_DIR, "merge" + i + ".sam").getAbsolutePath());
             }
             final File outputSam = new File(outputDir, TEST_BASE_NAME + ".sam");
             args.add("OUTPUT=" + outputSam.getAbsolutePath());
             args.add("METRICS_FILE=" + new File(outputDir, TEST_BASE_NAME + ".duplicate_metrics").getAbsolutePath());
+            args.add("ADD_PG_TAG_TO_READS=true");
             if (suppressPg) args.add("PROGRAM_RECORD_ID=null");
 
             // I generally prefer to call doWork rather than invoking the argument parser, but it is necessary
@@ -112,7 +110,7 @@ public class MarkDuplicatesTest extends AbstractMarkDuplicatesCommandLineProgram
             // the PG ID should be the same for both ends of a pair.
             final SamReader reader = SamReaderFactory.makeDefault().open(outputSam);
 
-            final Map<String, String> pgIdForReadName = new HashMap<String, String>();
+            final Map<String, String> pgIdForReadName = new HashMap<>();
             for (final SAMRecord rec : reader) {
                 final String existingPgId = pgIdForReadName.get(rec.getReadName());
                 final String thisPgId = rec.getStringAttribute(SAMTag.PG.name());
@@ -141,7 +139,7 @@ public class MarkDuplicatesTest extends AbstractMarkDuplicatesCommandLineProgram
             }
 
         } finally {
-            TestUtil.recursiveDelete(outputDir);
+            IOUtil.recursiveDelete(outputDir.toPath());
         }
     }
 
@@ -162,15 +160,15 @@ public class MarkDuplicatesTest extends AbstractMarkDuplicatesCommandLineProgram
     @DataProvider(name = "pgRecordChainingTest")
     public Object[][] pgRecordChainingTestDataProvider() {
         // Two test cases: One in which PG record generation is enabled, the other in which it is turned off.
-        final Map<String, List<ExpectedPnAndVn>> withPgMap = new HashMap<String, List<ExpectedPnAndVn>>();
+        final Map<String, List<ExpectedPnAndVn>> withPgMap = new HashMap<>();
         withPgMap.put("1AAXX.1.1", Arrays.asList(new ExpectedPnAndVn(TEST_BASE_NAME, null), new ExpectedPnAndVn(TEST_BASE_NAME, "1"), new ExpectedPnAndVn("bwa", "1")));
         withPgMap.put("1AAXX.2.1", Arrays.asList(new ExpectedPnAndVn(TEST_BASE_NAME, null), new ExpectedPnAndVn("bwa", "2")));
         withPgMap.put("1AAXX.3.1", Arrays.asList(new ExpectedPnAndVn(TEST_BASE_NAME, null)));
 
-        final Map<String, List<ExpectedPnAndVn>> suppressPgMap = new HashMap<String, List<ExpectedPnAndVn>>();
+        final Map<String, List<ExpectedPnAndVn>> suppressPgMap = new HashMap<>();
         suppressPgMap .put("1AAXX.1.1", Arrays.asList(new ExpectedPnAndVn(TEST_BASE_NAME, "1"), new ExpectedPnAndVn("bwa", "1")));
         suppressPgMap .put("1AAXX.2.1", Arrays.asList(new ExpectedPnAndVn("bwa", "2")));
-        suppressPgMap .put("1AAXX.3.1", new ArrayList<ExpectedPnAndVn>(0));
+        suppressPgMap .put("1AAXX.3.1", new ArrayList<>(0));
         return new Object[][] {
                 { false, withPgMap},
                 { true, suppressPgMap}
@@ -197,7 +195,7 @@ public class MarkDuplicatesTest extends AbstractMarkDuplicatesCommandLineProgram
         markDuplicates.PROGRAM_RECORD_ID = null;
         Assert.assertEquals(markDuplicates.doWork(), 0);
         Assert.assertEquals(markDuplicates.numOpticalDuplicates(), expectedNumOpticalDuplicates);
-        TestUtil.recursiveDelete(outputDir);
+        IOUtil.recursiveDelete(outputDir.toPath());
 
     }
 
@@ -215,8 +213,8 @@ public class MarkDuplicatesTest extends AbstractMarkDuplicatesCommandLineProgram
         tester.addMappedFragment(2, 41212324, false, "50M", DEFAULT_BASE_QUALITY);
         tester.addMappedFragment(2, 41212324, true, "50M", DEFAULT_BASE_QUALITY);
         final String barcodeTag = "BC";
-        for (final SAMRecord record : new IterableAdapter<SAMRecord>(tester.getRecordIterator())) {
-            record.setAttribute(barcodeTag, "Barcode1");
+        for (final SAMRecord record : new IterableAdapter<>(tester.getRecordIterator())) {
+            record.setAttribute(barcodeTag, "GACT");
         }
         tester.addArg("BARCODE_TAG=" + barcodeTag);
         tester.runTest();
@@ -228,21 +226,23 @@ public class MarkDuplicatesTest extends AbstractMarkDuplicatesCommandLineProgram
         tester.addMatePair("RUNID:1:1:15993:13361", 2, 41212324, 41212310, false, false, false, false, "33S35M", "19S49M", true, true, false, false, false, DEFAULT_BASE_QUALITY);
         tester.addMatePair("RUNID:2:2:15993:13362", 2, 41212324, 41212310, false, false, true, true, "33S35M", "19S49M", true, true, false, false, false, DEFAULT_BASE_QUALITY);
         final String barcodeTag = "BC";
-        for (final SAMRecord record : new IterableAdapter<SAMRecord>(tester.getRecordIterator())) {
+        for (final SAMRecord record : new IterableAdapter<>(tester.getRecordIterator())) {
             record.setAttribute(barcodeTag, "Barcode1");
         }
         tester.addArg("BARCODE_TAG=" + barcodeTag);
+        tester.addArg("ALLOW_NON_DNA_UMI=" + true);
         tester.runTest();
     }
 
     @Test
     public void testWithBarcodeDuplicate() {
         final AbstractMarkDuplicatesCommandLineProgramTester tester = getTester();
+        tester.getSamRecordSetBuilder().setReadLength(68);
         tester.addMatePair("RUNID:1:1:15993:13361", 2, 41212324, 41212310, false, false, false, false, "33S35M", "19S49M", true, true, false, false, false, DEFAULT_BASE_QUALITY);
         tester.addMatePair("RUNID:2:2:15993:13362", 2, 41212324, 41212310, false, false, true, true, "33S35M", "19S49M", true, true, false, false, false, DEFAULT_BASE_QUALITY);
         final String barcodeTag = "BC";
-        for (final SAMRecord record : new IterableAdapter<SAMRecord>(tester.getRecordIterator())) {
-            record.setAttribute(barcodeTag, "Barcode1");
+        for (final SAMRecord record : new IterableAdapter<>(tester.getRecordIterator())) {
+            record.setAttribute(barcodeTag, "ATGC");
         }
         tester.addArg("BARCODE_TAG=" + barcodeTag);
         tester.runTest();
@@ -251,6 +251,7 @@ public class MarkDuplicatesTest extends AbstractMarkDuplicatesCommandLineProgram
     @Test
     public void testWithBarcodeComplex() {
         final AbstractMarkDuplicatesCommandLineProgramTester tester = getTester();
+        tester.getSamRecordSetBuilder().setReadLength(68);
         final String readNameOne = "RUNID:1:1:15993:13361";
         final String readNameTwo = "RUNID:2:2:15993:13362";
         final String readNameThree = "RUNID:3:3:15993:13362";
@@ -261,12 +262,12 @@ public class MarkDuplicatesTest extends AbstractMarkDuplicatesCommandLineProgram
         tester.addMatePair(readNameThree, 2, 41212324, 41212310, false, false, false, false, "33S35M", "19S49M", true, true, false, false, false, DEFAULT_BASE_QUALITY);
 
         final String barcodeTag = "BC";
-        for (final SAMRecord record : new IterableAdapter<SAMRecord>(tester.getRecordIterator())) {
+        for (final SAMRecord record : new IterableAdapter<>(tester.getRecordIterator())) {
             if (record.getReadName().equals(readNameOne) || record.getReadName().equals(readNameTwo)) {
-                record.setAttribute(barcodeTag, "Barcode1");
+                record.setAttribute(barcodeTag, "AAAA");
             }
             else if (record.getReadName().equals(readNameThree)) {
-                record.setAttribute(barcodeTag, "Barcode2");
+                record.setAttribute(barcodeTag, "CCCC");
             }
         }
         tester.addArg("BARCODE_TAG=" + barcodeTag);
@@ -276,6 +277,7 @@ public class MarkDuplicatesTest extends AbstractMarkDuplicatesCommandLineProgram
     @Test
     public void testWithIndividualReadBarcodes() {
         final AbstractMarkDuplicatesCommandLineProgramTester tester = getTester();
+        tester.getSamRecordSetBuilder().setReadLength(68);
         final String readNameOne = "RUNID:1:1:15993:13361";
         final String readNameTwo = "RUNID:2:2:15993:13362";
         final String readNameThree = "RUNID:3:3:15993:13362";
@@ -288,16 +290,16 @@ public class MarkDuplicatesTest extends AbstractMarkDuplicatesCommandLineProgram
         final String barcodeTag = "BC";
         final String readOneBarcodeTag = "BX"; // want the same tag as the second end, since this is allowed
         final String readTwoBarcodeTag = "BX";
-        for (final SAMRecord record : new IterableAdapter<SAMRecord>(tester.getRecordIterator())) {
-            record.setAttribute(barcodeTag, "Barcode1"); // same barcode
+        for (final SAMRecord record : new IterableAdapter<>(tester.getRecordIterator())) {
+            record.setAttribute(barcodeTag, "ATC"); // same barcode
             if (record.getFirstOfPairFlag()) { // always the same value for the first end
-                record.setAttribute(readOneBarcodeTag, "readOne1");
+                record.setAttribute(readOneBarcodeTag, "ACA");
             }
             else { // second end
                 if (record.getReadName().equals(readNameOne) || record.getReadName().equals(readNameTwo)) {
-                    record.setAttribute(readTwoBarcodeTag, "readTwo1");
+                    record.setAttribute(readTwoBarcodeTag, "GTC");
                 } else if (record.getReadName().equals(readNameThree)) {
-                    record.setAttribute(readTwoBarcodeTag, "readTwo2");
+                    record.setAttribute(readTwoBarcodeTag, "CGA");
                 }
             }
         }
@@ -307,4 +309,71 @@ public class MarkDuplicatesTest extends AbstractMarkDuplicatesCommandLineProgram
 
         tester.runTest();
     }
+
+    @DataProvider(name = "testDuplexUmiDataProvider")
+    public Object[][] testDuplexUmiDataProvider() {
+        return new Object[][]{
+                {
+                        // Test case where UMIs are not duplex, but are the same.
+                        false,                                // Use duplex UMI (true), or single stranded UMI (false)
+                        Arrays.asList("ATC", "ATC"),          // UMIs
+                        Arrays.asList(4, 4),                  // First of pair alignment start
+                        Arrays.asList(10, 10),                // Second of pair alignment start
+                        Arrays.asList(false, true),           // Is duplicate
+                        Arrays.asList(false, false),          // Negative Strand Flag of first in pair
+                        Arrays.asList(true, true),            // Negative Strand Flag of second in pair
+                },{
+                        // Test case where UMIs are not duplex, but are different.  None of the reads should be duplicates.
+                        false,                                // Use duplex UMI (true), or single stranded UMI (false)
+                        Arrays.asList("ATC", "GCG"),          // UMIs
+                        Arrays.asList(4, 4),                  // First of pair alignment start
+                        Arrays.asList(10, 10),                // Second of pair alignment start
+                        Arrays.asList(false, false),          // Is duplicate
+                        Arrays.asList(false, false),          // Negative Strand Flag of first in pair
+                        Arrays.asList(true, true),            // Negative Strand Flag of second in pair
+                },
+                {
+                        // Two fragments, one top and bottom strand that are duplicates of each other.
+                        true,                                // Use duplex UMI (true), or single stranded UMI (false)
+                        Arrays.asList("AAA-GGG", "GGG-AAA"), // UMIs
+                        Arrays.asList(4, 10),                // First of pair alignment start
+                        Arrays.asList(10, 4),                // Second of pair alignment start
+                        Arrays.asList(false, true),          // Is duplicate
+                        Arrays.asList(false, true),          // Negative Strand Flag of first in pair
+                        Arrays.asList(true, false),          // Negative Strand Flag of second in pair
+                },
+                {
+                        // Two fragments, one top and bottom strand that are not duplicates of each other but may
+                        // naively appear to be duplicates because they have the same UMI (but not the same
+                        // top strand normalized UMI).
+                        true,                                // Use duplex UMI (true), or single stranded UMI (false)
+                        Arrays.asList("AAA-GGG", "AAA-GGG"), // UMIs
+                        Arrays.asList(4, 10),                // First of pair alignment start
+                        Arrays.asList(10, 4),                // Second of pair alignment start
+                        Arrays.asList(false, false),         // Is duplicate
+                        Arrays.asList(false, true),          // Negative Strand Flag of first in pair
+                        Arrays.asList(true, false),          // Negative Strand Flag of second in pair
+                },
+
+        };
+    }
+
+    @Test(dataProvider = "testDuplexUmiDataProvider")
+    public void testWithUMIs(final boolean duplexUmi, final List<String> UMIs, final List<Integer> alignmentStart1,
+                             final List<Integer> alignmentStart2, final List<Boolean> isDuplicate,
+                             final List<Boolean> isFirstNegativeStrand, final List<Boolean> isSecondNegativeStrand) {
+        final AbstractMarkDuplicatesCommandLineProgramTester tester = getTester();
+        tester.getSamRecordSetBuilder().setReadLength(10);
+
+        for (int k = 0; k < UMIs.size();k++) {
+            tester.addMatePair("RUNID" + k, 2, 2, alignmentStart1.get(k), alignmentStart2.get(k), false, false, isDuplicate.get(k), isDuplicate.get(k),
+                    "10M", "10M", isFirstNegativeStrand.get(k), isSecondNegativeStrand.get(k),
+                    false, false, false, DEFAULT_BASE_QUALITY, UMIs.get(k));
+        }
+        final String barcodeTag = "RX";
+
+        tester.addArg("BARCODE_TAG=" + barcodeTag);
+        tester.addArg("DUPLEX_UMI=" + duplexUmi);
+        tester.runTest();
+    }
 }
